<?php

/**
 * 调用 vecloud 模板分类, 列表, 提交渲染等 api 示例
 * 
 * @note 
 * 1. 该示例完整的演示了如何从 vecloud 获取模板, 提交渲染等功能, 用户根据自己的需要参考修改, 整合到自己的项目中
 * 2. 该示例为了配合前端 web demo 能够上传替换素材, 提供了获取 阿里云 oss 上传 token 的功能, 这个不属于 vecloud 的服务, 
 *      用户根据自己的业务去调整(比如用了七牛的存储, 自然使用七牛的 sdk)
 *
 * @note 文档参考 http://vesdk.com/docs/page_292.html
 * @note 文档参考 http://vesdk.com/docs/page_293.html
 *
 * */
include "./vesdk/Base.php";
include "./vesdk/Cpr.php";
include "./vesdk/Template.php";
include "./vendor/aliyuncs/oss-sdk-php/src/OSS/Core/OssException.php";
include "./vendor/aliyuncs/oss-sdk-php/src/OSS/OssClient.php";

use vesdk\Template;
use OSS\Core\OssException;
use OSS\OssClient;
use yii\web\Response;
use vesdk\Cpr;

ini_set("display_errors", "On");
error_reporting(E_ERROR);

class TemplateDemo
{
    public $template;

    public $cpr;

    public $config;

    public function __construct($config)
    {
        $this->template = new \vesdk\Template($config);
        $this->cpr      = new \vesdk\Cpr($config);
        $this->config   = $config;
    }

    public function getTemplateList($category_id)
    {
        return $this->template->getList($category_id);
    }

    public function getBalance()
    {
        return $this->template->getBalance();
    }

    public function getCategory()
    {
        return $this->template->getCategory();
    }

    public function getDetail($id)
    {
        return $this->template->getDetail($id);
    }

    public function getStatus($recordId)
    {
        return $this->cpr->getStatus($recordId);
    }


    /**
     * 获取json参数
     * @param string $get
     *
     * @return array|bool|mixed
     */
    public function getJsonParam($get = '')
    {
        $body = json_decode(file_get_contents('php://input'), true);

        if (empty($body)) {
            return empty($get) ? [] : false;
        }

        foreach ($body as &$string) {
            $string = is_string($string) ? htmlspecialchars(addslashes($string)) : $string;
        }


        return $get ? $body[$get] : $body;
    }

    /**
     * 提交模板渲染任务
     * 
     * @note notifyUrl 修改成客户自己的回调链接
     * 
     */
    public function startRender()
    {
        $replaceableJson = $this->getJsonParam("replaceableJson");
        $tplId           = $this->getJsonParam("tpl_id");
        $musicUrl        = $this->getJsonParam("music_url");
        $templateType    = $this->getJsonParam("template_type") ?: 1;
        $ret             = [];
        $post2cpr        = [];
        $randRecordId    = md5(time() . rand(100000, 999999));

        $post2cpr["musicUrl"]            = $musicUrl;
        $post2cpr["musicLoop"]           = true;
        $post2cpr["recordId"]            = (string)$randRecordId;
        $post2cpr["ossId"]               = $this->config["ossId"];
        $post2cpr["uploadKey"]           = $randRecordId . ".mp4";
        $post2cpr["bitrateControl"]      = (string)0.1; // php 7.1 以上会出现 json_encode 浮点精度的问题, 所以强制把浮点型数据转成 string, 防止 token 验证失败
                                                        // 参考: https://stackoverflow.com/questions/42981409/php7-1-json-encode-float-issue
        $post2cpr["notifyUrl"]           = "http://vesdk.com"; // 客户修改成自己的回调 url
        $post2cpr["tplId"]               = (string)$tplId;
        $post2cpr["isDepositedTemplate"] = true;
        $post2cpr["tplType"]             = $templateType;
        $post2cpr["replaceableJson"]     = $replaceableJson;
        $post2cpr["tplVersion"]          = 1;
        $post2cpr["waterMark"]           = "http://static.atvideo.cc/2021/04/16/13/49/watermark.png";
        $post2cpr["waterMarkPositionX"]  = 10;
        $post2cpr["waterMarkPositionY"]  = 10;

        try {
            $sdk = new \vesdk\Cpr($this->config);
        } catch (\Exception $e) {
            $ret["errormsg"] = ["提交失败:" . $e->getMessage()];
            $ret["errno"]    = 50000;

            return $ret;
        }

        $result = $sdk->request($post2cpr);
        if (!$result["data"]["orderId"]) {
            $ret["data"]     = $result;
            $ret["errormsg"] = ["提交失败"];
            $ret["errno"]    = 50001;
            return $ret;
        }
        $ret["data"]     = [
            "res"       => $result,
            "video_url" => $post2cpr["uploadKey"],
            "record_id" => $randRecordId,
            "order_id"  => $result["data"]["orderId"]
        ];
        $ret["errno"]    = 0;
        $ret["errormsg"] = "成功";

        return $ret;
    }
}

/**
 * 获取阿里云 OSS 上传 token, 客户根据自己的业务决定是否使用或者删除 
 * 
 * @note 如需使用更多 aliyun oss 的功能, 参考阿里云的官方文档
 * 
 */
class AliyunOssController
{
    private $accessId   = "";
    private $accessKey  = "";
    private $host       = "";


    public function __construct($config)
    {
        $this->accessId    = $config['accessId'];
        $this->accessKey = $config['accessKey'];
        $this->host         = $config['host'];
    }

    public function getOssSignature($dir)
    {
        $dir                  = (string)$dir;          // 用户上传文件时指定的前缀。
        $callback_param       = [
            'callbackBody'     => 'filename=${object}&size=${size}&mimeType=${mimeType}&height=${imageInfo.height}&width=${imageInfo.width}',
            'callbackBodyType' => "application/x-www-form-urlencoded"
        ];
        $callback_string      = json_encode($callback_param);
        $base64_callback_body = base64_encode($callback_string);
        $now                  = time();
        $expire               = 600;  //设置该policy超时时间是10s. 即这个policy过了这个有效时间，将不能访问。
        $end                  = $now + $expire;
        $expiration           = self::gmt_iso8601($end);
        //最大文件大小.用户可以自己设置
        $condition    = [0 => 'content-length-range', 1 => 0, 2 => 10485760000];
        $conditions[] = $condition;

        $arr                   = ['expiration' => $expiration, 'conditions' => $conditions];
        $policy                = json_encode($arr);
        $base64_policy         = base64_encode($policy);
        $string_to_sign        = $base64_policy;
        $signature             = base64_encode(hash_hmac('sha1', $string_to_sign, $this->accessKey, true));
        $response              = [];
        $response['accessid']  = $this->accessId;
        $response['host']      = $this->host;
        $response['policy']    = $base64_policy;
        $response['signature'] = $signature;
        $response['expire']    = $end;
        $response['callback']  = $base64_callback_body;
        $response['dir']       = $dir;  // 这个参数是设置用户上传文件时指定的前缀。

        return $response;
    }

    private static function gmt_iso8601($time)
    {
        $dtStr      = date("c", $time);
        $myDatetime = new \DateTime($dtStr);
        $expiration = $myDatetime->format(\DateTime::ISO8601);
        $pos        = strpos($expiration, '+');
        $expiration = substr($expiration, 0, $pos);
        return $expiration . "Z";
    }
}

/**
 * 任务配置信息, 从 console.vesdk.com 控制台获取
 * @note 所有字段都必须
 * */
$config = [
    "accessKey"    => "", //在控制台配置的秘钥
    "accessSecret" => "", //在控制台配置的秘钥
    "productId"    => "", //在控制台查看产品id
    "ossId"        => "", // 在控制台的OSSID
];

/**
 * 阿里云 OSS 配置信息, 用于生成前端需要的上传token,
 * 从阿里云 OSS 控制台获取
 * 
 */
$aliyunOssConfig = [
    "accessId"   => "", // 阿里云 oss 的 access key id
    "accessKey"  => "", // 阿里云 oss 的 access key secret
    "host"       => ""  // 阿里云 oss 的 bucket 域名
];


$demo = new TemplateDemo($config);

//获取您的余额 （用于模板租用和流量使用的，并非云渲染时长）
//$response = $demo->getBalance();

//获取模板分类
//$response = $demo->getCategory();

//获取模板详情
//$id = "";
//$response = $demo->getDetail();


//======================================================================
// 从这里开始, 根据 act 返回对应的信息
// 实际上是在这里调用了 vecloud 的 api, 直接返回 vecloud api 的数据 
// 根据自己的业务处理这块的逻辑

header('Content-type: application/json');

switch ($_GET["act"]) {
    case "category";
        $res = $demo->getCategory();
        echo json_encode($res);
        break;
    case "detail":
        //获取模板详情
        $id       = $_GET["id"];
        $response = $demo->getDetail($id);
        echo json_encode($response);
        break;
    case "get_upload_key":
        //获取阿里云上传秘钥
        //如不是使用阿里云，请自行接入SDK
        $aliyunOss = new AliyunOssController($aliyunOssConfig);
        $sign      = $aliyunOss->getOssSignature("");
        echo json_encode($sign);
        break;

    case "start_render":
        $ret = $demo->startRender();
        echo json_encode($ret);
        break;

    case "get_status":
        $recordId       = $_GET["record_id"];
        $ret = $demo->getStatus($recordId);
        echo json_encode($ret);
        break;
    default:
        // 获取云平台存储的模板列表
        $category_id = $_GET["category_id"] ?: "";
        $res = $demo->getTemplateList($category_id);
        echo json_encode($res);
        break;
}
