<?php
namespace vesdk;

class Base
{
    protected $accessKey = "your accessKey";
    protected $accessSecret = "your accessSecret";
    protected $productId = "your productId";
    protected $ossId = "your ossId";
    protected $remoteHosts = [
        "vecloud.atvideo.cc",
        "vecloud0.atvideo.cc",
        "vecloud1.atvideo.cc",
    ];


    public function __construct($config)
    {
        if (!$config["accessKey"])
        {
            throw new \Exception("accessKey not defined");
        }
        if (!$config["accessSecret"])
        {
            throw new \Exception("accessSecret not defined");
        }
        if (!$config["productId"])
        {
            throw new \Exception("productId not defined");
        }
        if (!$config["ossId"])
        {
            throw new \Exception("ossId not defined");
        }

        $this->accessKey    = $config["accessKey"];
        $this->accessSecret = $config["accessSecret"];
        $this->productId    = $config["productId"];
        $this->ossId        = $config["ossId"];
    }


    protected function post($data, $path)
    {
        $hosts_num = count($this->remoteHosts);
        foreach ($this->remoteHosts as $key => $remoteHost)
        {
            $url           = $remoteHost . $path;
            $token         = self::renderToken($data, $this->productId, $this->accessKey, $this->accessSecret);
            $Authorization = $this->productId . ":" . $this->accessKey . ":" . $token;

            $headerArray = [
                "Authorization:$Authorization",
            ];
            $curl        = curl_init();
            curl_setopt($curl, CURLOPT_URL, $url);
            curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
            curl_setopt($curl, CURLOPT_TIMEOUT, 5);
            curl_setopt($curl, CURLOPT_POST, 1);
            curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($data, JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES));
            curl_setopt($curl, CURLOPT_HTTPHEADER, $headerArray);
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
            $output   = curl_exec($curl);
            $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
            curl_close($curl);

            if ($key < $hosts_num && $httpCode !== 200)
            {
                continue;
            }

            if ($httpCode !== 200)
            {
                return ["errno" => $httpCode, "errormsg" => $output, "data" => []];
            }

            return json_decode($output, true); //json
        }

    }

    protected function checkParam($data, $rules)
    {

        foreach ($rules as $key => $rule)
        {
            $rule = explode("|", $rule);
            if ($rule[0] === "required")
            {
                switch ($rule[1])
                {
                    case "int" :
                        if (!is_int($data[ $key ]))
                        {
                            return ["res" => false, "type" => "int", "data" => $data[ $key ], "key" => $key];
                        }
                        break;
                    case "string" :
                        if (!is_string($data[ $key ]))
                        {
                            return ["res" => false, "type" => "string", "data" => $data[ $key ], "key" => $key];
                        }
                        if (@$rule[2] && preg_match("/" . $rule[2] . "/", $data[ $key ]))
                        {
                            return ["res" => false, "type" => $rule[2], "data" => $data[ $key ], "key" => $key];
                        }
                        break;
                    case "array" :
                        if (!is_array($data[ $key ]))
                        {
                            return ["res" => false, "type" => "array", "data" => $data[ $key ], "key" => $key];
                        }
                        break;
                    case "float" :
                        if (!is_float($data[ $key ]))
                        {
                            return ["res" => false, "type" => "array", "data" => $data[ $key ], "key" => $key];
                        }
                        break;
                    default :
                        break;
                }
            }
        }

        return ["res" => true];
    }

    protected function renderToken($rawData, $product_id, $access_key, $access_secret)
    {
        $encryptString = md5(
            json_encode($rawData, JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES) . "&&" . $product_id . "&&" . $access_key . "&&" . md5($access_secret)
        );

        return $encryptString;
    }
}
